/*
 * @(#)MacOSMenuBarUI.java	1.00 99/02/22
 *
 * Copyright 1999 by Luca Lutterotti,
 * Department of Material Engineering, University of Trento
 * via Mesiano, 77 - 38050 Trento, Italy
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Luca Lutterotti ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with the author.
 */

package it.unitn.ing.swing.plaf.macos;

import javax.swing.*;
import javax.swing.event.*;
import java.lang.reflect.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.plaf.basic.*;

import javax.swing.border.*;
import javax.swing.plaf.*;


/**
 * A default L&F implementation of MenuBarUI.  This implementation
 * is a "combined" view/controller.
 *
 * @version 1.00 02/22/99
 * @author Luca Lutterotti
 * @author David Ayman Shamma (modifications)
 */
 
public class MacOSMenuBarUI extends MenuBarUI implements MacOSMenuBarUtilities {
    
  protected JMenuBar              menuBar = null;
  protected ContainerListener     containerListener;
  protected ChangeListener        changeListener;
  private PropertyChangeListener  propertyChangeListener;
  protected MenuBar								macMenuBar;
  
  private boolean wasVisible = true;		
	protected static Vector elementInQueue = new Vector(0, 2);
	public static Vector currentMenuBarList = new Vector(0, 1);

  public static ComponentUI createUI(JComponent x) {
		return new MacOSMenuBarUI();
  }

  public void installUI(JComponent c) {
		menuBar = (JMenuBar) c;

 		installDefaults();
    installListeners();
    installKeyboardActions();
 	}

 	protected void installDefaults() {
		if (menuBar.getLayout() == null ||
	    	menuBar.getLayout() instanceof UIResource)
	    menuBar.setLayout(new DefaultMenuLayout(menuBar, BoxLayout.X_AXIS));

		menuBar.setOpaque(true);
		LookAndFeel.installBorder(menuBar,"MenuBar.border");
		LookAndFeel.installColorsAndFont(	menuBar,
					      											"MenuBar.background",
					      											"MenuBar.foreground",
					      											"MenuBar.font");
  }

  protected void installListeners() {
    containerListener = createContainerListener();
    changeListener = createChangeListener();
    propertyChangeListener = createPropertyChangeListener();
	
   	for (int i = 0; i < menuBar.getMenuCount(); i++) {
      JMenu menu = menuBar.getMenu(i);
	    if (menu!=null)
				menu.getModel().addChangeListener(changeListener);        
		}
		menuBar.addContainerListener(containerListener);
    menuBar.addPropertyChangeListener(propertyChangeListener);
  }

  protected void installKeyboardActions() {
		menuBar.registerKeyboardAction(
								new TakeFocus(menuBar),
							 	KeyStroke.getKeyStroke(	KeyEvent.VK_F10,
				       													0,
				       													false),
								JComponent.WHEN_IN_FOCUSED_WINDOW);
  } 

  public void uninstallUI(JComponent c) {
  	
  	if (menuBar != null) {
			removeMenuBar();

  		uninstallDefaults();
  		uninstallListeners();
  		uninstallKeyboardActions();

			menuBar = null;
		}
	}

  protected void uninstallDefaults() {
		LookAndFeel.uninstallBorder(menuBar);
  }

  protected void uninstallListeners() {
		menuBar.removeContainerListener(containerListener);

  	for (int i = 0; i < menuBar.getMenuCount(); i++) {
	    JMenu menu = menuBar.getMenu(i);
	    if (menu !=null)
				menu.getModel().removeChangeListener(changeListener);
		}

		containerListener = null;
		changeListener = null;
    propertyChangeListener = null;
	}

	protected void uninstallKeyboardActions() {
		menuBar.unregisterKeyboardAction(
		       		KeyStroke.getKeyStroke(	KeyEvent.VK_F10,
					      											0,
					      											false));
  }

	protected ContainerListener createContainerListener() {
		return new ContainerHandler();
	}

	protected ChangeListener createChangeListener() {
		return new ChangeHandler();
	}

  private PropertyChangeListener createPropertyChangeListener() {
    return new PropertyChangeHandler();
  }

	private class ChangeHandler implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
	    int i,c;
	    for(i=0,c = menuBar.getMenuCount() ; i < c ; i++) {
				JMenu menu = menuBar.getMenu(i);
				if(menu !=null && menu.isSelected()) {
		    	menuBar.getSelectionModel().setSelectedIndex(i);
		    	break;
				}
	    }
		}
  }

    /*
     * This PropertyChangeListener is used to adjust the default layout
     * manger when the menuBar is given a right-to-left ComponentOrientation.
     * This is a hack to work around the fact that the DefaultMenuLayout
     * (BoxLayout) isn't aware of ComponentOrientation.  When BoxLayout is
     * made aware of ComponentOrientation, this listener will no longer be
     * necessary.
     */
    private class PropertyChangeHandler implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            String name = e.getPropertyName();
            if( name.equals("componentOrientation")
                && (menuBar.getLayout() instanceof UIResource) )
            {
                if( isLeftToRight(menuBar) ) {
                    menuBar.setLayout(new DefaultMenuLayout(menuBar,BoxLayout.X_AXIS));
                } else {
                    menuBar.setLayout(new RightToLeftMenuLayout());
                }
            } else if (name.equals("menubaradded")) {
  						installMacMenu();
							consumeQueue();
            } else if (name.equals("UI")) {
							Object oldValue = e.getOldValue();
							Object newValue = e.getNewValue();
							if (oldValue != null && newValue != oldValue && newValue == MacOSMenuBarUI.this) {
  							installMacMenu();
								consumeQueue();
							} else if (newValue != MacOSMenuBarUI.this && oldValue == MacOSMenuBarUI.this) {
								removeMenuBar();
							}
            }
        }
    }
    
    /*
     * Convenience function for determining ComponentOrientation.  Helps us
     * avoid having Munge directives throughout the code.
     */
    static boolean isLeftToRight( Component c ) {
        /*if[JDK1.2]
        return c.getComponentOrientation().isLeftToRight();
        else[JDK1.2]*/
        return true;
        /*end[JDK1.2]*/
    }
  public Dimension getPreferredSize(JComponent c) {
		if ( isThePrincipalMenuBar(menuBar) )
			return new Dimension(0,0);
		else
    	return null;
  }

  public Dimension getMinimumSize(JComponent c) {
		if ( isThePrincipalMenuBar(menuBar) )
			return new Dimension(0,0);
		else
    	return null;
  }

  public Dimension getMaximumSize(JComponent c) {
		if ( isThePrincipalMenuBar(menuBar) )
			return new Dimension(0,0);
		else
    	return null;
  }
  
	private class ContainerHandler implements ContainerListener {
		public void componentAdded(ContainerEvent e) {
	    Component c = e.getChild();
	    if (c instanceof JMenu) {
				((JMenu)c).getModel().addChangeListener(changeListener);
	    	updateMacMenu((JMenu) c);
	    }
		}
		public void componentRemoved(ContainerEvent e) {
	    Component c = e.getChild();
	    if (c instanceof JMenu)
				((JMenu)c).getModel().removeChangeListener(changeListener);
			if (c instanceof JComponent)
	    	updateMacMenuForRemoving((JComponent) c);
		}
  }


  private static class TakeFocus implements ActionListener {
		JMenuBar menuBar;

    TakeFocus(JMenuBar menuBar) {
	  	this.menuBar = menuBar;
		}
	
		public void actionPerformed(ActionEvent e) {
    	MenuSelectionManager defaultManager = MenuSelectionManager.defaultManager();
	 		MenuElement me[];
	 		MenuElement subElements[];
	  	JMenu menu = menuBar.getMenu(0);
	 		if (menu!=null) {
		  	me = new MenuElement[3];
		  	me[0] = menuBar;
		  	me[1] = menu;
		  	me[2] = menu.getPopupMenu();
		  	defaultManager.setSelectedPath(me);
	   	}
		}
		
		public boolean isEnabled() {
	  	return menuBar.isEnabled();
		}
	}
    
    private static class RightToLeftMenuLayout
        extends FlowLayout implements UIResource
    {
        private RightToLeftMenuLayout() {
            super(3/*FlowLayout.LEADING*/,0,0);
        }
    }

  public void installMacMenu() {
  	if (macMenuBar == null) {
			if (!isOnJDialog(menuBar))
				isThePrincipalMenuBarAndCache(menuBar);
				
  		if (isThePrincipalMenuBar(menuBar)) {
				addMacMenuBar(menuBar);
  			wasVisible = menuBar.isVisible();
  			if (wasVisible)
					menuBar.setVisible(false);
				macMenuBar = new MenuBar();
				Container parent = getRootParent(menuBar);

				if (parent != null && parent instanceof Frame)
						((Frame) parent).setMenuBar(macMenuBar);
			}
  	}
  }
  
  protected void updateMacMenu(JMenu c) {
		try {
			ComponentUI mui = c.getUI();
			((MacOSMenuBarUtilities) mui).installMacMenu();
		} catch (ClassCastException ce) {
			c.updateUI();
			ComponentUI mui = c.getUI();
			((MacOSMenuBarUtilities) mui).installMacMenu();
		}
		
  }
  
  protected void updateMacMenuForRemoving(JComponent c) {
		try {
			ComponentUI mui = MacOSMenuUI.getUI(c);
			((MacOSMenuBarUtilities) mui).removeMacMenu(macMenuBar);
		} catch (ClassCastException ce) {
			c.updateUI();
			ComponentUI mui = MacOSMenuUI.getUI(c);
			((MacOSMenuBarUtilities) mui).removeMacMenu(macMenuBar);
		} catch (Exception e) {
		}
		
  }
  
  protected void removeMenuBar() {
  	if (menuBar != null && macMenuBar != null) {
			Container parent = getRootParent(menuBar);
			if (parent != null && parent instanceof Frame)
				((Frame) parent).setMenuBar(null);
			
			removeMacMenuBar(menuBar);
  		if (wasVisible)
				menuBar.setVisible(true);
			macMenuBar = null;
  	}
  }

  public void addToMacMenuBar(MenuItem item) {
	  if (macMenuBar != null && item != null)
	  	if (item instanceof Menu) {
	  		if (((Menu) item).getActionCommand().startsWith("Help")) {
	  			macMenuBar.setHelpMenu((Menu) item);
//	  		System.out.println("Add "+item.getActionCommand()+ " to: " + macMenuBar + " for help");
	  		}
	  		else
	  			macMenuBar.add((Menu) item);
//	  		System.out.println("Add "+item.getActionCommand()+ " to: " + macMenuBar);
	  	}
  }
  	
  public void removeMacMenu(Menu parentMenu) {
  }
  	
  public void removeMacMenu(MenuBar parentMenu) {
  }
  	
  public MenuItem getMacMenuItem() {
  	return null;
  }
  
  public static void addToQueue(Object menuitem, Object macmenuitem) {
  	elementInQueue.addElement(menuitem);
  	elementInQueue.addElement(macmenuitem);
  }
  
  // ayman's addition MacOSMenuBarUI::shiftQueue(JComponent c)
  public static void shiftQueue(JComponent c) {
		if (c != null && elementInQueue.contains((JComponent) c)) {
			int i = elementInQueue.indexOf((JComponent) c);
			elementInQueue.addElement(elementInQueue.elementAt(i));
			elementInQueue.addElement(elementInQueue.elementAt(i+1));
			elementInQueue.removeElementAt(i+1);
			elementInQueue.removeElementAt(i);
		}
		consumeQueue();
  }
		
  public static void consumeQueue() {
  	int i = 0;
  	int actualSize = elementInQueue.size();
  	while (i < elementInQueue.size() - 1) {
  		Object menuitem = elementInQueue.elementAt(i);
  		Object macmenuitem = elementInQueue.elementAt(i+1);
	  	MacOSMenuBarUtilities me = MacOSMenuItemUI.getParentUI((JComponent) menuitem);
	  	if (me != null) {
	    	me.addToMacMenuBar((MenuItem) macmenuitem);
	    	elementInQueue.removeElementAt(i+1);
	    	elementInQueue.removeElementAt(i);
//	    	System.out.println("Consumed: " + menuitem.toString());
	    } else {
//	    	System.out.println("Not consumed: " + menuitem.toString());
	    	i += 2;
	    }
		}
		if (elementInQueue.size() != actualSize)
			consumeQueue();
  }

  public static void addMacMenuBar(JMenuBar menuBar) {
  	currentMenuBarList.addElement(menuBar);
  }

  public static void removeMacMenuBar(JMenuBar menuBar) {
  	currentMenuBarList.removeElement(menuBar);
  }

  public static JMenuBar getMacMenuBar(int index) {
  	if (index >= 0 && index < currentMenuBarList.size())
  		return (JMenuBar) currentMenuBarList.elementAt(index);
  	else
  		return null;
  }
  
  public static MenuElement getComponentParentInMenuBar(Component c) {
  	for (int i = currentMenuBarList.size() - 1; i >= 0; i--) {
  		JMenuBar menubar = getMacMenuBar(i);
	    for(int j = menubar.getMenuCount() - 1 ; j >= 0 ; j--) {
				JMenu menu = menubar.getMenu(j);
				if(menu == c)
		    	return menubar;
		    MenuElement me = getComponentParentInSubMenu(menu, c);
		    if (me != null)
		    	return me;
	    }
  	}
  	return null;
  }

  public static MenuElement getComponentParentInSubMenu(JMenu menu, Component c) {
    if (menu == null) return null;            // mjh - for jEdit
  	int numberElements = menu.getMenuComponentCount();
  	for (int i = 0; i < numberElements; i++) {
  		Component menuitem = menu.getMenuComponent(i);
			if (menuitem == c)
		    return menu;
		  if (menuitem instanceof JMenu) {
		  	MenuElement me = getComponentParentInSubMenu((JMenu) menuitem, c);
		    if (me != null)
		    	return me;
		  }
  	}
  	return null;
  }

  protected static boolean isOnJDialog(JMenuBar menuBar) {
		Container parent = getRootParent(menuBar);
		if (parent != null)
			if (parent instanceof Dialog)
				return true;
		return false;
  }
  
  protected static Container getRootParent(JMenuBar menuBar) {
		Container parent = menuBar.getParent();
		while (parent != null && !(parent instanceof Frame) && !(parent instanceof Dialog))
			parent = parent.getParent();
		
		return parent;
  }
  
  private boolean isPrincipalMenuBar = false;

  boolean isThePrincipalMenuBarAndCache(JMenuBar menuBar) {
  	isPrincipalMenuBar = true;
		Container parent = getRootParent(menuBar);
  	for (int i = 0, c = currentMenuBarList.size(); i < c; i++) {
  		JMenuBar menub = (JMenuBar) currentMenuBarList.elementAt(i);
			Container secondparent = getRootParent(menub);
  		if (secondparent == parent) {
  			isPrincipalMenuBar = false;
  			break;
  		}
  	}
  	return isPrincipalMenuBar;
  }
  	
  public boolean isThePrincipalMenuBar(JMenuBar menuBar) {
  	return isPrincipalMenuBar;
  }
  	
}
