package it.unitn.ing.swing.plaf.macos;

import java.awt.*;
import java.awt.event.*;

import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;

import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;


/**
 * MacOSOptionPaneUI implementation
 *
 * @version 1.00 22/01/2001
 * @author Daniel Bobbert <dabo@coli.uni-sb.de>
 */

public class MacOSOptionPaneUI extends BasicOptionPaneUI {

    public static ComponentUI createUI( JComponent c )
    {
        return new MacOSOptionPaneUI();
    }


    protected Container createButtonArea() {
    	Object buttons[] = getButtons();
    	boolean onlyStrings = true;
    	
    	for (int i=0;i<buttons.length; i++)
    		if (!(buttons[i] instanceof String))
    			onlyStrings = false;

        JPanel bottom = new JPanel();
        bottom.setBorder(UIManager.getBorder("OptionPane.buttonAreaBorder"));
        if (onlyStrings)
        	bottom.setLayout(new MacButtonAreaLayout(true, 6, getIcon()));
        else
			bottom.setLayout(new ButtonAreaLayout(true, 6));
		
		addButtonComponents(bottom, buttons, getInitialValueIndex());
		return bottom;
    }

    protected boolean getSizeButtonsToSameWidth() {
		return false;
    }

    /**
     * ButtonAreaLayout acts similiar to FlowLayout. It lays out all
     * components from left to right. If syncAllWidths is true, the widths
     * of each component will be set to the largest preferred size width.
     *
     * This inner class is marked &quot;public&quot; due to a compiler bug.
     * This class should be treated as a &quot;protected&quot; inner class.
     * Instantiate it only within subclasses of BasicOptionPaneUI.
     */  
    public static class MacButtonAreaLayout extends ButtonAreaLayout {
    	private Icon icon;
    	private final int minButtonWidth = 70;
    
		public MacButtonAreaLayout(boolean syncAllWidths, int padding, Icon icon) {
			super(syncAllWidths, padding);
			this.icon = icon;
		    centersChildren = false;
		}

	    public void setCentersChildren(boolean newValue) {
		    // don't want to center
		}

		public void layoutContainer(Container container) {
		    Component[]      children = container.getComponents();

		    if (children == null || children.length == 0)
		    	return;		    

			int               numChildren = children.length;
			Dimension[]       sizes = new Dimension[numChildren];
			int               counter;
			int               yLocation = container.getInsets().top;
		    int 			  leftWidth = 0, rightWidth = 0, height = 0;

		    children = (Component[]) children.clone();
			for (counter = 2; counter < numChildren; counter++) {
				if (children[counter] instanceof JButton) {
					JButton b = (JButton) children[counter];
					if (b.getText() != null ? b.getText().equals(UIManager.getString("OptionPane.cancelButtonText")) : false) {
						Component c = children[counter];
						children[counter] = children[1];
						children[1] = c;
						break;
					}
				}
			}

		    for (counter = 0; counter < numChildren; counter++) {
				sizes[counter] = children[counter].getPreferredSize();
				if (sizes[counter].width < minButtonWidth)
					sizes[counter].width = minButtonWidth;
				height = Math.max(height, sizes[counter].height);
			}

			if (numChildren >= 2) {
				int max = Math.max(sizes[0].width, sizes[1].width);
				sizes[0].width = max;
				sizes[1].width = max;
			}


		    for(counter = 2; counter < numChildren; counter++)
				leftWidth += sizes[counter].width;
				
			if (numChildren > 2)
				leftWidth += ((numChildren - 3) * padding);

		    rightWidth += sizes[0].width;
				
			if (numChildren > 1)
				rightWidth += (sizes[1].width + padding);


		    int          xOffset = padding;
		    int          xLocation = 0;


			// Align buttons with Text, so leave space under icon		    
			if (icon != null)
				xLocation += new JLabel(icon).getPreferredSize().width + 15;

			for (counter = numChildren-1; counter >=2; counter--) {
				children[counter].setBounds(xLocation, yLocation,
					 sizes[counter].width, height);
				xLocation += xOffset + sizes[counter].width;
			}
			
			xLocation = container.getSize().width - sizes[0].width;
			children[0].setBounds(xLocation, yLocation,
					 sizes[0].width, height);
			
			if (numChildren > 1) {
				xLocation -= (sizes[0].width + xOffset);
				children[1].setBounds(xLocation, yLocation,
						 sizes[1].width, height);
			}			
		}

		public Dimension minimumLayoutSize(Container c) {
		    if (c != null) {
				Component[]       children = c.getComponents();

				if (children == null || children.length == 0)
					return new Dimension(0, 0);
				
			    Dimension     sizes[] = new Dimension[children.length];
			    int           numChildren = children.length;
			    int           counter;
			    Insets        cInsets = c.getInsets();
			    int           extraHeight = cInsets.top + cInsets.bottom;


				int        totalWidth = 0, height = 0;

				for (counter = 0; counter < numChildren; counter++) {
					sizes[counter] = children[counter].getPreferredSize();
					if (sizes[counter].width < minButtonWidth)
						sizes[counter].width = minButtonWidth;
				}

				if (numChildren >= 2) {
					int max = Math.max(sizes[0].width, sizes[1].width);
					sizes[0].width = max;
					sizes[1].width = max;
				}

				for(counter = 0; counter < numChildren; counter++){
				    height = Math.max(height, sizes[counter].height);
				    totalWidth += sizes[counter].width;
				}
				totalWidth += ((numChildren - 1) * padding);
				
				if (icon != null)
					totalWidth += new JLabel(icon).getPreferredSize().width + 15;
								
				return new Dimension(totalWidth, extraHeight + height);
		    }
		    return new Dimension(0, 0);
		}

    }


}