package it.unitn.ing.swing.plaf.macos;

/**
  * From macnative package to swing/plaf/macos - mjh 09.10.00
  **/

import  com.apple.mrj.jdirect.*;				// mjh 02.14.99

/**
 * <CODE>RectStruct</CODE> is a class which mimics the C struct <CODE>Rect</CODE> defined in <CODE>MacTypes.h</CODE><BR><BR>
 * This class contains a get and set method for each field in <CODE>Rect</CODE>.<BR><BR>
 * Using JDirect 2.0, a <CODE>RectStruct</CODE> can be passed to a native toolbox function
 * that expects a pointer to a <CODE>Rect</CODE> by using the <CODE>getByteArray()</CODE> method. 
 */
public class RectStruct extends ByteArrayStruct {

	/**
	 * Constructs an uninitialized <CODE>RectStruct</CODE>
	 */
	public RectStruct() {
		super(sizeOfRect);
	}

	/**
	 * Constructs a <CODE>RectStruct</CODE> and initializes it with the data in another <CODE>Struct</CODE>. 
	 * Useful when a C struct <CODE>Rect</CODE> is embedded in another C struct and you want to
	 * copy it out. 
	 * @param src the <CODE>Struct</CODE> containing the data to be copied
	 * @param offsetInSrc the offest in of the data in <CODE>src</CODE>
	 */
	public RectStruct(Struct src, int offsetInSrc) {
		super(sizeOfRect);
		byte[] bytes = src.getBytesAt(offsetInSrc, sizeOfRect);
		this.setBytesAt(0, bytes);
	}

	/**
	 * Used only by subclasses of <CODE>RectStruct</CODE>
	 */
	protected RectStruct(int size) {
		super(size);
	}

	/**
	 * in C: <CODE>short top</CODE>
	 * @return field <CODE>top</CODE> of <CODE>struct Rect</CODE>
	 */
	public final short getTop() {
		return getShortAt(0);
	}

	/**
	 * in C: <CODE>short top</CODE>
	 * @param top sets field <CODE>top</CODE> of <CODE>struct Rect</CODE>
	 */
	public final void setTop(short top) {
		setShortAt(0, top);
	}

	/**
	 * in C: <CODE>short left</CODE>
	 * @return field <CODE>left</CODE> of <CODE>struct Rect</CODE>
	 */
	public final short getLeft() {
		return getShortAt(2);
	}

	/**
	 * in C: <CODE>short left</CODE>
	 * @param left sets field <CODE>left</CODE> of <CODE>struct Rect</CODE>
	 */
	public final void setLeft(short left) {
		setShortAt(2, left);
	}

	/**
	 * in C: <CODE>short bottom</CODE>
	 * @return field <CODE>bottom</CODE> of <CODE>struct Rect</CODE>
	 */
	public final short getBottom() {
		return getShortAt(4);
	}

	/**
	 * in C: <CODE>short bottom</CODE>
	 * @param bottom sets field <CODE>bottom</CODE> of <CODE>struct Rect</CODE>
	 */
	public final void setBottom(short bottom) {
		setShortAt(4, bottom);
	}

	/**
	 * in C: <CODE>short right</CODE>
	 * @return field <CODE>right</CODE> of <CODE>struct Rect</CODE>
	 */
	public final short getRight() {
		return getShortAt(6);
	}

	/**
	 * in C: <CODE>short right</CODE>
	 * @param right sets field <CODE>right</CODE> of <CODE>struct Rect</CODE>
	 */
	public final void setRight(short right) {
		setShortAt(6, right);
	}

	/**
	 * Size of <CODE>struct Rect</CODE> in bytes
	 */
	public final static int sizeOfRect = 8;

	public long getValue() {
		return getLongAt(0);
	}
}
